"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
exports.__esModule = true;
var NodePath = require("node:path");
var utils_1 = require("../utils");
var churn_1 = require("./churn/churn");
var complexity_1 = require("./complexity");
var DEFAULT_CHURN = 1;
var DEFAULT_COMPLEXITY = 1;
var internal = { debug: (0, utils_1.buildDebugger)("statistics") };
var Statistics = /** @class */ (function () {
    function Statistics(path, churn, complexity) {
        this.path = path;
        this.churn = churn;
        this.complexity = complexity;
        this.directories = this.findDirectoriesForFile(path);
        this.score = this.churn * this.complexity;
    }
    Statistics.compute = function (options) {
        return __awaiter(this, void 0, void 0, function () {
            var churns, paths, complexities, statisticsForFiles, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        internal.debug("invoked with options: ".concat(JSON.stringify(options)));
                        internal.debug("using cwd: ".concat(process.cwd()));
                        return [4 /*yield*/, churn_1["default"].compute(options)];
                    case 1:
                        churns = _a.sent();
                        paths = Array.from(churns.keys());
                        return [4 /*yield*/, complexity_1["default"].compute(paths, options)];
                    case 2:
                        complexities = _a.sent();
                        statisticsForFiles = paths.map(Statistics.toStatistics(churns, complexities));
                        result = options.directories
                            ? Statistics.buildDirectoriesStatistics(statisticsForFiles)
                            : statisticsForFiles;
                        return [2 /*return*/, result.sort(sort(options.sort)).filter(limit(options.limit))];
                }
            });
        });
    };
    Statistics.toStatistics = function (churns, complexities) {
        return function (path) {
            var churn = churns.get(path) || DEFAULT_CHURN;
            var complexity = complexities.get(path) || DEFAULT_COMPLEXITY;
            return new Statistics(path, churn, complexity);
        };
    };
    Statistics.buildDirectoriesStatistics = function (statisticsForFiles) {
        var map = statisticsForFiles.reduce(function (map, statisticsForFile) {
            statisticsForFile.directories.forEach(function (directoryForFile) {
                computeStatisticsForDirectory(map, directoryForFile, statisticsForFile);
            });
            return map;
        }, new Map());
        return __spreadArray([], __read(map.values()), false);
        function computeStatisticsForDirectory(map, dir, statisticsForFile) {
            var statisticsForDir = map.get(dir);
            var churn = statisticsForFile.churn +
                (statisticsForDir ? statisticsForDir.churn : 0);
            var complexity = statisticsForFile.complexity +
                (statisticsForDir ? statisticsForDir.complexity : 0);
            map.set(dir, new Statistics(dir, churn, complexity));
        }
    };
    Statistics.prototype.findDirectoriesForFile = function (path) {
        var directories = [];
        var pathChunks = NodePath.parse(path).dir.split(NodePath.sep);
        pathChunks.forEach(function (chunk) {
            var parentDir = directories.slice(-1);
            var directory = parentDir.length
                ? parentDir + NodePath.sep + chunk
                : chunk;
            directories.push(directory);
        });
        return directories.filter(function (d) { return d.length > 0; });
    };
    Statistics.prototype.toState = function () {
        return {
            path: this.path,
            churn: this.churn,
            complexity: this.complexity,
            score: this.score
        };
    };
    return Statistics;
}());
exports["default"] = Statistics;
function limit(limit) {
    return function (statistics, i) { return !limit || i < limit; };
}
function sort(sort) {
    return function (statisticsA, statisticsB) {
        if (sort === "score" || sort === "ratio") {
            return statisticsB.score - statisticsA.score;
        }
        if (sort === "churn") {
            return statisticsB.churn - statisticsA.churn;
        }
        if (sort === "complexity") {
            return statisticsB.complexity - statisticsA.complexity;
        }
        if (sort === "file") {
            var fileAPath = statisticsA.path;
            var fileBPath = statisticsB.path;
            return fileAPath.localeCompare(fileBPath);
        }
        return 0;
    };
}
